@@ -1,5 +1,7 @@
 package Dancer::Handler;
-
+our $AUTHORITY = 'cpan:SUKRIA';
+# ABSTRACT: Dancer request handler
+$Dancer::Handler::VERSION = '1.3202';
 use strict;
 use warnings;
 use Carp;
@@ -30,7 +32,6 @@ sub get_handler {
     if ($ENV{'PLACK_ENV'}) {
         Dancer::Logger::core("PLACK_ENV is set (".$ENV{'PLACK_ENV'}.") forcing PSGI handler");
         setting('apphandler'  => 'PSGI');
-        setting('environment' => $ENV{'PLACK_ENV'});
     }
 
     # if Plack is detected or set by conf, use the PSGI handler
@@ -70,9 +71,7 @@ sub handle_request {
 
     Dancer::Cookies->init;
 
-    if (Dancer::Config::setting('auto_reload')) {
-        Dancer::App->reload_apps;
-    }
+    Dancer::App->reload_apps if Dancer::Config::setting('auto_reload');
 
     render_request($request);
     return $self->render_response();
@@ -138,7 +137,7 @@ sub init_request_headers {
     Dancer::SharedData->headers($psgi_headers);
 }
 
-# render a PSGI-formated response from a response built by
+# render a PSGI-formatted response from a response built by
 # handle_request()
 sub render_response {
     my $self     = shift;
@@ -155,8 +154,10 @@ sub render_response {
             $response->header( 'Content-Type' => "$ctype; charset=$charset" )
               if $ctype !~ /$charset/;
         }
-        $response->header( 'Content-Length' => length($content) )
-          if !defined $response->header('Content-Length');
+        if (!defined $response->header('Content-Length')) {
+            use bytes; # turn off character semantics
+            $response->header( 'Content-Length' => length($content) );
+        }
         $content = [$content];
     }
     else {
@@ -171,7 +172,7 @@ sub render_response {
       if ( defined Dancer::SharedData->request
         && Dancer::SharedData->request->is_head() );
 
-    # drop content AND content_length if reponse is 1xx or (2|3)04
+    # drop content AND content_length if response is 1xx or (2|3)04
     if ($response->status =~ (/^[23]04$/ )) {
         $content = [''];
         $response->header('Content-Length' => 0);
@@ -194,7 +195,7 @@ sub render_response {
 
 sub _is_text {
     my ($content_type) = @_;
-    return $content_type =~ /(x(?:ht)?ml|text|json|javascript)/;
+    return $content_type =~ /(\bx(?:ht)?ml\b|text|json|javascript)/;
 }
 
 # Fancy banner to print on startup
@@ -208,3 +209,30 @@ sub print_banner {
 sub dance { (shift)->start(@_) }
 
 1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Dancer::Handler - Dancer request handler
+
+=head1 VERSION
+
+version 1.3202
+
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
